from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.visual_search_service.file_list import FileList
from media_platform.service.media_platform_request import MediaPlatformRequest


class FindSimilarImagesRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', f'{base_url}/visual_search', FileList)

        self.image_url = None
        self.collection_id = None

    def set_image_url(self, image_url: str) -> FindSimilarImagesRequest:
        self.image_url = image_url
        return self

    def set_collection_id(self, collection_id: str) -> FindSimilarImagesRequest:
        self.collection_id = collection_id
        return self

    def execute(self):
        self.url = f'{self.url}/collections/{self.collection_id}/search'
        return super().execute()

    def _params(self) -> dict:
        return {
            'imageUrl': self.image_url,
        }



from __future__ import annotations

from media_platform.lang.serialization import Deserializable
from media_platform.metadata.security.antivirus import ThreatDetectionMetadata
from media_platform.service.file_descriptor import FileDescriptor


class MediaType:
    undefined = ''
    image = 'image'
    video = 'video'
    audio = 'audio'
    font = 'font'


class FileMetadata(Deserializable):
    def __init__(self, media_type: str, file_descriptor: FileDescriptor, basic: Deserializable = None,
                 threat_detection: ThreatDetectionMetadata = None):
        self.media_type = media_type
        self.file_descriptor = file_descriptor
        self.basic = basic
        self.threat_detection = threat_detection

    @classmethod
    def deserialize(cls, data: dict) -> FileMetadata:
        file_descriptor = FileDescriptor.deserialize(data['fileDescriptor'])
        threat_detection = ThreatDetectionMetadata.deserialize(data.get('threatDetection')) \
            if data.get('threatDetection') else None

        return FileMetadata(MediaType.undefined, file_descriptor, threat_detection=threat_detection)



from __future__ import annotations

from typing import cast

from media_platform.job.job_group import JobGroup


class ScanFileJobGroup(JobGroup):

    @classmethod
    def deserialize(cls, data: dict) -> ScanFileJobGroup:
        return cast(ScanFileJobGroup, super().deserialize(data))



from __future__ import annotations

import datetime

from media_platform.lang import datetime_serialization
from media_platform.lang.serialization import Serializable, Deserializable


class AntivirusMetadata(Serializable, Deserializable):
    def __init__(self, infected: bool, signature: str, timestamp: datetime.datetime):
        self.infected = infected
        self.signature = signature
        self.timestamp = timestamp

    @classmethod
    def deserialize(cls, data: dict) -> AntivirusMetadata:
        return AntivirusMetadata(
            data.get('infected'),
            data.get('signature'),
            datetime_serialization.deserialize(data['scanTimestamp'])
        )

    def serialize(self) -> dict:
        return {
            'infected': self.infected,
            'signature': self.signature,
            'scanTimestamp': self.timestamp
        }


class ThreatDetectionMetadata(Serializable, Deserializable):
    def __init__(self, antivirus: AntivirusMetadata = None):
        self.antivirus = antivirus

    @classmethod
    def deserialize(cls, data: dict) -> ThreatDetectionMetadata:
        return ThreatDetectionMetadata(AntivirusMetadata.deserialize(data.get('antivirus')))

    def serialize(self) -> dict:
        return {
            'antivirus': self.antivirus.serialize()
        }



from __future__ import annotations

from media_platform.job.job import Job
from media_platform.job.job_type import JobType
from media_platform.job.specification import Specification


class IndexImageSpecification(Specification):
    @classmethod
    def deserialize(cls, data):
        return IndexImageSpecification()

    def serialize(self) -> dict:
        return {}


class IndexImageJob(Job):
    type = JobType.index_image
    specification_type = IndexImageSpecification



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.index_image_job import IndexImageJob
from media_platform.service.callback import Callback
from media_platform.service.media_platform_request import MediaPlatformRequest
from media_platform.service.source import Source


class IndexImageRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'POST', f'{base_url}/visual_search', IndexImageJob)

        self.source = None
        self.callback = None
        self.collection_id = None

    def set_source(self, source: Source) -> IndexImageRequest:
        self.source = source
        return self

    def set_collection_id(self, collection_id: str) -> IndexImageRequest:
        self.collection_id = collection_id
        return self

    def set_callback(self, callback: Callback):
        self.callback = callback
        return self

    def execute(self):
        self.url = f'{self.url}/collections/{self.collection_id}/index'
        return super().execute()

    def _params(self) -> dict:
        return {
            'source': self.source.serialize(),
            'jobCallback': self.callback.serialize() if self.callback else None,
            'specification': None,
        }



from __future__ import annotations

from media_platform.metadata.file_metadata import FileMetadata, MediaType
from media_platform.metadata.image.image_basic import ImageBasic
from media_platform.metadata.image.image_features import ImageFeatures
from media_platform.metadata.security.antivirus import ThreatDetectionMetadata
from media_platform.service.file_descriptor import FileDescriptor


class ImageFileMetadata(FileMetadata):
    def __init__(self, file_descriptor: FileDescriptor, basic: ImageBasic = None, features: ImageFeatures = None,
                 threat_detection: ThreatDetectionMetadata = None):
        super().__init__(MediaType.image, file_descriptor, basic, threat_detection)

        self.features = features

    @classmethod
    def deserialize(cls, data: dict) -> ImageFileMetadata:
        if data['mediaType'] != MediaType.image:
            raise ValueError('not image metadata')

        file_descriptor = FileDescriptor.deserialize(data['fileDescriptor'])
        basic = ImageBasic.deserialize(data.get('basic')) if data.get('basic') else None
        features = ImageFeatures.deserialize(data.get('features')) if data.get('features') else None
        threat_detection = ThreatDetectionMetadata.deserialize(data.get('threatDetection')) \
            if data.get('threatDetection') else None

        return ImageFileMetadata(file_descriptor, basic, features, threat_detection)



from __future__ import annotations

from typing import Optional, Dict

from media_platform.job.job_type import JobType
from media_platform.job.result.job_result import JobResult
from media_platform.service.file_descriptor import FileDescriptor


class TranscodeResult(JobResult):
    type = JobType.transcode

    def __init__(self, code: int = None, message: str = None, file_descriptor: FileDescriptor = None,
                 master_ffmpeg_command: str = None, error_class: str = None):
        super().__init__(code, message)
        self.file_descriptor = file_descriptor
        self.master_ffmpeg_command = master_ffmpeg_command
        self.error_class = error_class

    @classmethod
    def deserialize(cls, data: Optional[Dict]) -> Optional[TranscodeResult]:
        if data is None:
            return None

        payload_data = data.get('payload') or {}
        file_descriptor_data = payload_data.get('file')
        file_descriptor = FileDescriptor.deserialize(file_descriptor_data) if file_descriptor_data else None

        return cls(data['code'], data['message'], file_descriptor, payload_data.get('masterFFMpegCommand'),
                   payload_data.get('errorClass'))

    def serialize(self) -> Dict:
        return {
            'code': self.code,
            'message': self.message,
            'payload': self._serialize_payload()
        }

    def _serialize_payload(self) -> Dict:
        payload = {}

        if self.file_descriptor:
            payload = self.file_descriptor.serialize()
            # backwards compatibility
            payload['file'] = self.file_descriptor.serialize()

        if self.master_ffmpeg_command:
            payload['masterFFMpegCommand'] = self.master_ffmpeg_command

        if self.error_class:
            payload['errorClass'] = self.error_class

        return payload or None



from __future__ import annotations
from typing import List, Dict

from media_platform.lang.serialization import Deserializable
from media_platform.service.file_descriptor import FileDescriptor


class FileList(Deserializable):
    def __init__(self, files: [FileDescriptor]):
        self.files = files

    @classmethod
    def deserialize(cls, data: List[Dict]) -> FileList:
        return FileList([FileDescriptor.deserialize(f) for f in data])



from __future__ import annotations

from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.file_descriptor import FileDescriptor
from media_platform.service.media_platform_request import MediaPlatformRequest


class DeleteFileRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client: AuthenticatedHTTPClient, base_url: str):
        super().__init__(authenticated_http_client, 'DELETE', base_url + '/files', None)
        self.path = None

    def set_path(self, path: str) -> DeleteFileRequest:
        self.path = path
        return self

    def execute(self):
        return super().execute()

    def validate(self):
        FileDescriptor.path_validator(self.path)

    def _params(self) -> dict:
        return {
            'path': self.path
        }



from media_platform.http_client.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.media_platform_service import MediaPlatformService
from media_platform.service.visual_search_service.index_image_request import IndexImageRequest
from media_platform.service.visual_search_service.find_similar_images_request import FindSimilarImagesRequest


class VisualSearchService(MediaPlatformService):
    def __init__(self, domain: str, authenticated_http_client: AuthenticatedHTTPClient):
        super(VisualSearchService, self).__init__(domain, authenticated_http_client)

    def index_image_request(self) -> IndexImageRequest:
        return IndexImageRequest(self._authenticated_http_client, self._base_url)

    def find_similar_images_request(self) -> FindSimilarImagesRequest:
        return FindSimilarImagesRequest(self._authenticated_http_client, self._base_url)
